
import React, {useEffect, useState, useRef, useLayoutEffect} from "react";
import * as d3 from "d3";
import { AxisX, AxisY } from '../../components/d3js/Axis';
import {ApiService} from "../../components/d3js/ApiService";

const textLabel = <>
    <tspan x="0" dy="1.2em">Nombre</tspan>
    <tspan x="0" dy="1.2em">de</tspan>
    <tspan x="0" dy="1.2em">ventes</tspan>
</>

const groups = [
    { group: "day", label: "jours" },
    { group: "week", label: "semaines" },
    { group: "month", label: "mois" },
    { group: "year", label: "années" },
];
const defaults = {
    groupBy: groups[2],
    dateStart: new Date("2018-01-01"),
    dateEnd: new Date("2019-01-01")
};
const minDate = "2018-01-01";
const maxDate = "2022-01-01";

export default function Series() {
    const Y_MAX = 500;
    const X_MAX = 1250;
    const MARGIN_LEFT = 150;
    const MARGIN_TOP = 50;

    const [items, setItems] = useState<{date: string, value: number}[]>([])
    const [xTicks, setXTicks] = useState<{ value: Date; xOffset: number; }[]>([])
    const [yTicks, setYTicks] = useState<{value: number, yOffset: number}[]>([])
    const [data,setData] = useState<{ date: Date; xOffset: number; amount: number, width: number, height: number}[]>([])
    const [divWidth, setDivWidth] = useState(-1);
    const [groupeBy, setGroupeBy] = useState(defaults.groupBy.group);
    const [dateDebut, setDateDebut] = useState(defaults.dateStart)
    const [dateFin, setDateFin] = useState(defaults.dateEnd);

    //Gère la largeur du graphique en fonction de la taille de la fenêtre
    useEffect(() => {
        const resizeGraph = () => {
            let div = document.querySelector<HTMLDivElement>("#salesGraph");
            if(div)
                setDivWidth(div.offsetWidth);
        }
        window.addEventListener("resize", resizeGraph)
        resizeGraph();
    }, [])

    //Met a jour les données lorsque la taille de la fenêtre change, ou lorsque l'utilisateur change la date de début / de fin
    useEffect(() => {
        ApiService.getAmounts(dateDebut, dateFin, groupeBy)
            .then(({ amounts }) => {
                setItems(amounts);
            });
    }, [dateDebut, dateFin, groupeBy])

    //Met a jour le graph lorsque les données changent
    useEffect(() => {
        if(!items || items.length === 0)
            return;

        const xScale = d3.scaleTime()
            .domain([dateDebut, dateFin])
            .nice()
            .range([50, (divWidth > 0?divWidth-250:X_MAX)])

        //On récupère les ticks de xScale, en enlevant le dernier élément en trop
        let tmpTicks = xScale.ticks()
            .map(value => ({
                value,
                xOffset: xScale(value)
            }))

        setXTicks(tmpTicks)

        const sortedItems = [...items]
        sortedItems.sort((a,b) => {return a.value - b.value})

        const yScale = d3.scaleLinear()
            .domain([sortedItems[sortedItems.length - 1].value, 0])
            .nice()
            .range([0, Y_MAX])

        setYTicks(yScale.ticks()
            .map(value => ({
                value,
                yOffset: yScale(value)
            })))

        setData(items.map(item => {
            let startDate = new Date(item.date)
            let endDate = new Date(startDate)
            endDate.setMonth(endDate.getMonth()+1);
            return {
                date: startDate,
                xOffset: xScale(startDate),
                amount: item.value,
                height: Y_MAX - yScale(item.value),
                width: xScale(endDate) - xScale(startDate)
            }
        }));
    }, [items, divWidth])

    return (
        <div id="salesGraph">
            <div className="d-flex flex-column align-items-center">
                <div>
                    <p>ventes par
                        <select
                            onChange={(e) => setGroupeBy(e.target.value)}
                            defaultValue={groupeBy}
                        >
                            {groups.map(({group, label}) => (
                                <option value={group}>{label}</option>
                            ))}
                        </select>
                    </p>
                </div>
                <div className="d-flex align-items-baseline">
                    <p>entre le</p>
                    <input className="ms-2" type="date"
                        value={dateDebut.toISOString().substring(0, dateDebut.toISOString().indexOf("T"))} onChange={(e) => setDateDebut(new Date(e.target.value))}
                        min={minDate} max={maxDate} />
                </div>
                <div className="d-flex align-items-baseline">
                    <p>et le</p>
                    <input className="ms-2" type="date"
                        value={dateFin.toISOString().substring(0, dateFin.toISOString().indexOf("T"))} onChange={(e) =>setDateFin(new Date(e.target.value))}
                        min={minDate} max={maxDate} />
                </div>
            </div>
            <svg id="graph"
                width="100%"
                height={Y_MAX + MARGIN_TOP + 30}
            >
                {/* Définition du motif d'hachure pour les rectangles */}
                <defs>
                    <pattern
                            id="diagonalHatch"
                            patternUnits="userSpaceOnUse" 
                            width="4" 
                            height="8" 
                            patternTransform="rotate(-45 2 2)"
                        >
                            <path d="M -1,2 l 6,0" stroke="red" strokeWidth=".5"/>
                    </pattern>
                </defs>
                <AxisX ticks={xTicks} transform={`translate(${MARGIN_LEFT}, ${Y_MAX + MARGIN_TOP})`}/>
                <AxisY ticks={yTicks} transform={`translate(${MARGIN_LEFT}, ${MARGIN_TOP})`} textLabel={textLabel}/>
                <BarChart data={data} transform={`translate(${MARGIN_LEFT}, ${MARGIN_TOP})`} yMax={Y_MAX}/>
            </svg>
        </div>
    )
}

/**
 * Fonction dessinant l'histogramme
 * @param param0 
 * @returns 
 */
function BarChart({data, transform, yMax}: {data:{ date: Date; xOffset: number; amount: number, width: number, height: number}[], transform:string, yMax:number}){
    return (
        <g
            key="barChart"
            transform={transform}
        >
            {data.map(({ date, xOffset, width, height }) => (
                <g
                    key={"data:"+date.toISOString()}
                    transform={`translate(${xOffset}, ${yMax - height})`}
                >
                    <rect 
                        width={width}
                        height={height}
                        stroke="red"
                        fill="url(#diagonalHatch)"
                    />
                </g>
            ))}
        </g>
    )
}